<?php
require_once __DIR__.'/../../api/nachos.php';
require_once __DIR__.'/../../api/lib/request.php';
require_once __DIR__.'/../../api/lib/response.php';
require_once __DIR__.'/../../api/lib/error_response.php';


/**
 * @MockRequest a mocked request object
 * @param $body Array with the post body
 * @param $request_method The http method
 */
class MockRequest extends Api_Request {
    private $body;
    private $request_method;

    public function __construct($body, $request_method) {
        $this->body = $body;
        $this->request_method = $request_method;
    }

    public function get_body() {
        return $this->body;
    }

    public function get_method() {
        return $this->request_method;
    }
}

if (!class_exists('op5log')) {
    /**
     * Dummy op5 log
     */
    class op5log {
        public static function instance($name) {
            return new self;
        }

        public function log($type, $desc) {
            return null;
        }
    }
}

if (!class_exists('op5config')) {
    /**
     * Dummy op5 config
     */
    class op5config {
        public static function instance() {
            return new self;
        }

        public function getConfig($dummy) {
            return "dummy";
        }

        public function getPathForNamespace($dummy)
        {
            return "/etc/dummy.yml";
        }
    }
}

class NachosTest extends PHPUnit_Framework_TestCase {
    public function test_full_export_valid() {
        $request = new MockRequest(array('export_type'=>'full_export'), 'POST');

        # Mock the method that does the actual CURL request to nachos
        $MockNachos = $this->getMockBuilder('\Nachos')->setMethods(array('query_nachos'))->getMock();
        $MockNachos->expects(
            $this->once())->method('query_nachos')->will(
            $this->returnValue(array(
                'response' => '{"creation_time":"2018-05-29 07:32:42.544838","export_type":"full_export","id":' .
                    '"7705eb78-d0b6-4b15-8016-a165c2db8052","status":"pending","user":"monitor"}',
                'http_code' => 202
            )));

        $response = $MockNachos->handle_request($request);
        $body = $response->get_body();

        $this->assertInstanceOf('Api_Response', $response);
        $this->assertSame(202, $response->get_code());

        $this->assertSame('2018-05-29 07:32:42.544838', $body->creation_time);
        $this->assertSame('full_export', $body->export_type);
        $this->assertSame('7705eb78-d0b6-4b15-8016-a165c2db8052', $body->id);
        $this->assertSame('pending', $body->status);
        $this->assertSame('monitor', $body->user);
    }

    public function test_user_export_valid() {
        $request = new MockRequest(array('export_type'=>'user_export'), 'POST');

        $MockNachos = $this->getMockBuilder('\Nachos')->setMethods(array('query_nachos'))->getMock();
        $MockNachos->expects(
            $this->once())->method('query_nachos')->will(
            $this->returnValue(array(
                'response' => '{"creation_time":"2018-05-29 07:32:42.544838","export_type":"user_export","id":' .
                    '"7705eb78-d0b6-4b15-8016-a165c2db8052","status":"pending","user":"monitor"}',
                'http_code' => 202
            )));

        $response = $MockNachos->handle_request($request);
        $body = $response->get_body();

        $this->assertInstanceOf('Api_Response', $response);
        $this->assertSame(202, $response->get_code());

        $this->assertSame('2018-05-29 07:32:42.544838', $body->creation_time);
        $this->assertSame('user_export', $body->export_type);
        $this->assertSame('7705eb78-d0b6-4b15-8016-a165c2db8052', $body->id);
        $this->assertSame('pending', $body->status);
        $this->assertSame('monitor', $body->user);
    }

    public function test_missing_export_type() {
        $request = new MockRequest(array(), 'POST');
        $MockNachos = $this->getMockBuilder('\Nachos')->setMethods(array('query_nachos'))->getMock();

        # Api_Error_Response returns the error message to the requester, but doesn't seem to set a message in the
        # actual exception
        $this->setExpectedException('Api_Error_Response', '', 400);
        $MockNachos->handle_request($request);
    }

    public function test_invalid_export_type() {
        $request = new MockRequest(array('export_type'=>'grandmother_export'), 'POST');
        $MockNachos = $this->getMockBuilder('\Nachos')->setMethods(array('query_nachos'))->getMock();

        $this->setExpectedException('Api_Error_Response', '', 400);
        $MockNachos->handle_request($request);
    }

    public function test_nachos_unresponsive() {
        $request = new MockRequest(array('export_type'=>'full_export'), 'POST');
        $MockNachos = $this->getMockBuilder('\Nachos')->setMethods(array('query_nachos'))->getMock();
        $MockNachos->expects(
            $this->once())->method('query_nachos')->will(
            $this->returnValue(array(
                'response' => '',
                'http_code' => 0
            )));

        $this->setExpectedException('Api_Error_Response', '', 500);
        $MockNachos->handle_request($request);
    }

    public function test_invalid_http_method() {
        $request = new MockRequest(array('export_type'=>'full_export'), 'PUT');
        $MockNachos = $this->getMockBuilder('\Nachos')->setMethods(array('query_nachos'))->getMock();

        $this->setExpectedException('Api_Error_Response', '', 405);
        $MockNachos->handle_request($request);
    }

    public function test_get_latest_export() {
        $request = new MockRequest(array(), 'GET');

        $MockNachos = $this->getMockBuilder('\Nachos')->setMethods(array('query_nachos'))->getMock();
        $MockNachos->expects(
            $this->once())->method('query_nachos')->will(
            $this->returnValue(array(
                'response' => '{"creation_time":"2018-05-31T07:25:23.000000","export_type":"full_export","id":' .
                    '"1ed3cfa0-7b9c-4d8c-9617-4136b819c87f","status":"success","status_details":[{"details":"backup ' .
                    'complete","progress":1,"state":"backup"},{"details":"","progress":1,"state":"config_generation"},'.
                    '{"details":"skip","progress":1,"state":"verification"},{"details":"","progress":1,"state":' .
                    '"commit"}],"user":"monitor"}',
                'http_code' => 200
            )));

        $response = $MockNachos->handle_request($request);
        $body = $response->get_body();

        $this->assertInstanceOf('Api_Response', $response);
        $this->assertSame(200, $response->get_code());

        $this->assertSame('2018-05-31T07:25:23.000000', $body->creation_time);
        $this->assertSame('full_export', $body->export_type);
        $this->assertSame('1ed3cfa0-7b9c-4d8c-9617-4136b819c87f', $body->id);
        $this->assertSame('success', $body->status);
        $this->assertSame('monitor', $body->user);
        $this->assertEquals(
            json_decode('[{"details":"backup ' .
                'complete","progress":1,"state":"backup"},{"details":"","progress":1,"state":"config_generation"},'.
                '{"details":"skip","progress":1,"state":"verification"},{"details":"","progress":1,"state":' .
                '"commit"}]'),
            $body->status_details);
    }

    public function test_get_latest_export_when_no_exports_exist() {
        $request = new MockRequest(array(), 'GET');

        $MockNachos = $this->getMockBuilder('\Nachos')->setMethods(array('query_nachos'))->getMock();
        $MockNachos->expects(
            $this->once())->method('query_nachos')->will(
            $this->returnValue(array(
                'response' => '{"description":"No export jobs could be found in the database","name":"no_exports_found_error"}',
                'http_code' => 404
            )));

        $response = $MockNachos->handle_request($request);
        $body = $response->get_body();

        $this->assertInstanceOf('Api_Response', $response);
        $this->assertSame(404, $response->get_code());

        $this->assertSame('No export jobs could be found in the database', $body->description);
        $this->assertSame('no_exports_found_error', $body->name);
    }
}
